@@ -1,27 +1,25 @@
-## Copyright 2015-2017 Tom Brown (FIAS), Jonas Hoersch (FIAS)
 
-## This program is free software; you can redistribute it and/or
-## modify it under the terms of the GNU General Public License as
-## published by the Free Software Foundation; either version 3 of the
-## License, or (at your option) any later version.
+## Copyright 2015-2021 PyPSA Developers
 
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
+## You can find the list of PyPSA Developers at
+## https://pypsa.readthedocs.io/en/latest/developers.html
 
-## You should have received a copy of the GNU General Public License
-## along with this program.  If not, see <http://www.gnu.org/licenses/>.
+## PyPSA is released under the open source MIT License, see
+## https://github.com/PyPSA/PyPSA/blob/master/LICENSE.txt
 
 """Graph helper functions, which are attached to network and sub_network
 """
 
+__author__ = "PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html"
+__copyright__ = ("Copyright 2015-2021 PyPSA Developers, see https://pypsa.readthedocs.io/en/latest/developers.html, "
+                 "MIT License")
+
 # Functions which will be attached to network and sub_network
 
 import scipy as sp
 import numpy as np
 
-from .descriptors import OrderedGraph
+from .descriptors import OrderedGraph, get_active_assets
 
 def graph(network, branch_components=None, weight=None, inf_weight=False):
     """
@@ -89,7 +87,8 @@ def graph(network, branch_components=None, weight=None, inf_weight=False):
 
     return graph
 
-def adjacency_matrix(network, branch_components=None, busorder=None, weights=None):
+def adjacency_matrix(network, branch_components=None, investment_period=None,
+                     busorder=None, weights=None):
     """
     Construct a sparse adjacency matrix (directed)
 
@@ -133,11 +132,19 @@ def adjacency_matrix(network, branch_components=None, busorder=None, weights=Non
     weight_vals = []
     for c in network.iterate_components(branch_components):
         if c.ind is None:
-            sel = slice(None)
-            no_branches = len(c.df)
+            if investment_period is None:
+                sel = slice(None)
+            else:
+                active = get_active_assets(network, c.name, investment_period)
+                sel = c.df.loc[active].index
         else:
-            sel = c.ind
-            no_branches = len(c.ind)
+            if investment_period is None:
+                sel = c.ind
+            else:
+                active = get_active_assets(network, c.name, investment_period, network.snapshots)
+                sel = c.ind & c.df.loc[active].index
+
+        no_branches = len(c.df.loc[sel])
         bus0_inds.append(busorder.get_indexer(c.df.loc[sel, "bus0"]))
         bus1_inds.append(busorder.get_indexer(c.df.loc[sel, "bus1"]))
         weight_vals.append(np.ones(no_branches)
